---@class ns
local ns = select(2, ...)

---@class FullTextSearchContext
---@field index FullTextWordIndex
local FullTextSearchContextPrototype = {}

---@param items SearchItem[]
local function CreateFullTextSearchContext(items)
	local context = setmetatable({}, { __index = FullTextSearchContextPrototype })
	local index = ns.FullTextWordIndex.Create()
	for _, item in next, items do
		index:AddString(item, item.name)
		index:AddString(item, item.category)
		if item.extraSearchText then
			index:AddString(item, item.extraSearchText)
		end
	end
	index:Index()
	context.index = index
	return context
end

---@param query string
---@return SearchContextItem[]
function FullTextSearchContextPrototype:Search(query)
	if query == "" then return {} end
	local weightedResults = self.index:Search(query)
	local results = self:ScoreWeightedResults(weightedResults)

	return results
end

---@param weightedResults table<SearchItem, number>
function FullTextSearchContextPrototype:ScoreWeightedResults(weightedResults)
	local results = {}
	for value, weight in next, weightedResults do
		results[#results + 1] = {
			item = value,
			score = weight - 50000, -- TODO configurable or something
		}
	end

	return results
end

local export = { Create = CreateFullTextSearchContext }
if ns then
	ns.FullTextSearchContext = export
end
return export
